#ifndef LANGSCAN_JAVA_H
#define LANGSCAN_JAVA_H

#include <stdlib.h>

#define LANGSCAN_JAVA_TOKEN_LIST \
  LANGSCAN_JAVA_TOKEN(preproc_beg) \
  LANGSCAN_JAVA_TOKEN(preproc_end) \
  LANGSCAN_JAVA_TOKEN(character) \
  LANGSCAN_JAVA_TOKEN(integer) \
  LANGSCAN_JAVA_TOKEN(floating) \
  LANGSCAN_JAVA_TOKEN(string) \
  LANGSCAN_JAVA_TOKEN(ident) \
  LANGSCAN_JAVA_TOKEN(punct) \
  LANGSCAN_JAVA_TOKEN(comment) \
  LANGSCAN_JAVA_TOKEN(space)

typedef enum {
  langscan_java_eof = 0,
#define LANGSCAN_JAVA_TOKEN(name) langscan_java_##name,
  LANGSCAN_JAVA_TOKEN_LIST
#undef LANGSCAN_JAVA_TOKEN
} langscan_java_token_t;

typedef struct {
  int beg_lineno;
  int beg_columnno;
  int beg_byteno;
  int end_lineno;
  int end_columnno;
  int end_byteno;
  int eof;
  char *text;
  int leng;
  size_t (*user_read)(void **user_data_p, char *buf, size_t maxlen);
  void *user_data;
} langscan_java_lex_extra_t;

typedef struct langscan_java_tokenizer_tag {
  langscan_java_lex_extra_t *extra;
  void *scanner;
} langscan_java_tokenizer_t;

typedef size_t (*user_read_t)(void **user_data_p, char *buf, size_t maxlen);

langscan_java_tokenizer_t *langscan_java_make_tokenizer(user_read_t user_read, void *user_data);
langscan_java_token_t langscan_java_get_token(langscan_java_tokenizer_t *tokenizer);
void langscan_java_free_tokenizer(langscan_java_tokenizer_t *tokenizer);

user_read_t langscan_java_tokenizer_get_user_read(langscan_java_tokenizer_t *tokenizer);
void *langscan_java_tokenizer_get_user_data(langscan_java_tokenizer_t *tokenizer);

const char *langscan_java_token_name(langscan_java_token_t token);
#define langscan_java_curtoken_beg_lineno(tokenizer) ((tokenizer)->extra->beg_lineno)
#define langscan_java_curtoken_beg_columnno(tokenizer) ((tokenizer)->extra->beg_columnno)
#define langscan_java_curtoken_beg_byteno(tokenizer) ((tokenizer)->extra->beg_byteno)
#define langscan_java_curtoken_end_lineno(tokenizer) ((tokenizer)->extra->end_lineno)
#define langscan_java_curtoken_end_columnno(tokenizer) ((tokenizer)->extra->end_columnno)
#define langscan_java_curtoken_end_byteno(tokenizer) ((tokenizer)->extra->end_byteno)
#define langscan_java_curtoken_text(tokenizer) ((tokenizer)->extra->text)
#define langscan_java_curtoken_leng(tokenizer) ((tokenizer)->extra->leng)

void langscan_java_extract_functions(langscan_java_tokenizer_t *);

#endif
