#ifndef LANGSCAN_PYTHON_H
#define LANGSCAN_PYTHON_H

#include <stdlib.h>

#define LANGSCAN_PYTHON_TOKEN_LIST \
  LANGSCAN_PYTHON_TOKEN(integer) \
  LANGSCAN_PYTHON_TOKEN(floating) \
  LANGSCAN_PYTHON_TOKEN(imaginary) \
  LANGSCAN_PYTHON_TOKEN(string) \
  LANGSCAN_PYTHON_TOKEN(ident) \
  LANGSCAN_PYTHON_TOKEN(punct) \
  LANGSCAN_PYTHON_TOKEN(comment) \
  LANGSCAN_PYTHON_TOKEN(space)

typedef enum {
  langscan_python_eof = 0,
#define LANGSCAN_PYTHON_TOKEN(name) langscan_python_##name,
  LANGSCAN_PYTHON_TOKEN_LIST
#undef LANGSCAN_PYTHON_TOKEN
} langscan_python_token_t;

typedef struct {
  int beg_lineno;
  int beg_columnno;
  int beg_byteno;
  int end_lineno;
  int end_columnno;
  int end_byteno;
  int eof;
  char *text;
  int leng;
  size_t (*user_read)(void **user_data_p, char *buf, size_t maxlen);
  void *user_data;
} langscan_python_lex_extra_t;

typedef struct langscan_python_tokenizer_tag {
  langscan_python_lex_extra_t *extra;
  void *scanner;
} langscan_python_tokenizer_t;

typedef size_t (*user_read_t)(void **user_data_p, char *buf, size_t maxlen);

langscan_python_tokenizer_t *langscan_python_make_tokenizer(user_read_t user_read, void *user_data);
langscan_python_token_t langscan_python_get_token(langscan_python_tokenizer_t *tokenizer);
void langscan_python_free_tokenizer(langscan_python_tokenizer_t *tokenizer);

user_read_t langscan_python_tokenizer_get_user_read(langscan_python_tokenizer_t *tokenizer);
void *langscan_python_tokenizer_get_user_data(langscan_python_tokenizer_t *tokenizer);

const char *langscan_python_token_name(langscan_python_token_t token);
#define langscan_python_curtoken_beg_lineno(tokenizer) ((tokenizer)->extra->beg_lineno)
#define langscan_python_curtoken_beg_columnno(tokenizer) ((tokenizer)->extra->beg_columnno)
#define langscan_python_curtoken_beg_byteno(tokenizer) ((tokenizer)->extra->beg_byteno)
#define langscan_python_curtoken_end_lineno(tokenizer) ((tokenizer)->extra->end_lineno)
#define langscan_python_curtoken_end_columnno(tokenizer) ((tokenizer)->extra->end_columnno)
#define langscan_python_curtoken_end_byteno(tokenizer) ((tokenizer)->extra->end_byteno)
#define langscan_python_curtoken_text(tokenizer) ((tokenizer)->extra->text)
#define langscan_python_curtoken_leng(tokenizer) ((tokenizer)->extra->leng)

void langscan_python_extract_functions(langscan_python_tokenizer_t *);

#endif
