require 'fileutils'
include FileUtils

def gnu_make?
  message = IO.popen("make --version").read
  /^GNU/.match(message)
end

module TestUtil
  FOO_FILES   = ["bar.c", "bar.h", "foo.c", "Makefile" ].sort
  FOO_C_FILES = FOO_FILES.find_all {|x| /\.[ch]$/.match(x) }
  FOO_SYMBOLS = ["bar", "foo", "main", "printf"]
  FOO_PACKAGE = "foo-0.1"
  FOO_TGZ     = File.join("foo", FOO_PACKAGE + ".tar.gz")

  @@make_options = if gnu_make?
                     "--quiet --no-print-directory"
                   else
                     ""
                   end

  def add_package (config)
    importer = Importer.new(config)
    importer.add_archive(FOO_TGZ)
    importer.finish
  end

  def make_db (config)
    remove_db(config)
    make_archives
    importer = Gonzui::Importer.new(config)
    importer.add_archive(FOO_TGZ)
    importer.finish
  end

  def remove_db (config)
    rm_rf(config.db_directory)
  end

  def make_dist_tree
    system("cd foo; make #{@@make_options} -f Makefile.foo dist-tree")
  end

  def make_clean
    system("cd foo; make #{@@make_options} -f Makefile.foo clean")
  end

  def make_archives
    require_command("zip")
    require_command("tar")
    require_command("gzip")
    require_command("bzip2")
    system("cd foo; make #{@@make_options} -f Makefile.foo dist")
    system("cd foo; make #{@@make_options} -f Makefile.foo dist-poor")
  end
end

if ENV['MAKEFLAGS'] # be quiet in a make process.
  ARGV.unshift("--verbose=s")
end
