#! /usr/bin/env ruby
$LOAD_PATH.unshift("..")
require 'test/unit'
require 'gonzui'
require 'test-util'
include Gonzui

class TextTokenizerTest < Test::Unit::TestCase
  include TestUtil

  def _test_collect (tokenizer)
    list = []
    tokenizer.each {|word, pos, lineno_offset|
      assert(word.is_a?(String))
      assert(pos.is_a?(Integer))
      assert(lineno_offset.is_a?(Integer))
      list.push([word, pos, lineno_offset])
    }
    return list
  end

  def _test_with_words (words, delim)
    text = words.join(delim)
    tokenizer = TextTokenizer.new(text)
    list = _test_collect(tokenizer)

    pos = 0
    lineno = 0
    words.each_with_index {|word, i|
      assert_equal(word, list[i][0])
      assert_equal(pos, list[i][1])
      assert_equal(lineno, list[i][2])
      pos += word.length + delim.length
      lineno += delim.count("\n")
    }
  end

  def _test_text (text, n)
    tokenizer = TextTokenizer.new(text)
    list = _test_collect(tokenizer)
    assert_equal(n, list.length)
  end

  def test_each
    words = ["foo", "bar", "baz"]
    _test_with_words(words, " ")
    _test_with_words(words, "\n")
    _test_with_words(words, ", ")
    _test_with_words(words, ",\n")
    _test_with_words(words, "!#%&\n\n\n@@")

    _test_text("foo_bar", 1)
    _test_text("あ", 1)  # single multi-byte character
    _test_text("あい", 2) # two multi-byte characters
  end
end
